: $Id: delay.mod,v 1.2 2010/06/30 21:51:26 ted Exp ted $

COMMENT
A delay line implemented with a ring buffer.
The delayed variable is sampled at fixed intervals that are 
controlled by the event delivery system, so this mechanism will
"work" with fixed time step or adaptive integration.

Pointers specified at the hoc level are used to connect 
this mechanism's input and output variables to hoc.

Fixed parameter (mod file must be edited and recompiled):
MAXNUM  size of the ring buffer (max number of samples that can be stored)

Variable parameters (can be specified from hoc without recompiling):
invl  sample interval
num   number of samples in the buffer

Computed value:
del   delay = num*invl


Caveats:

1.  Adaptive integration will treat the sampled variable as 
a staircase function, not a time continuous function.
This has two consequences--
  artifacts may be caused by the abrupt transitions of 
  the output values
and 
  constancy of the output variable between transitions
  means that the approximation to a time delay of a 
  continuous function has only first order accuracy.

2.  Models that involve delays raise issues with regard to 
initialization, and conservation (of charge or material).
For example, if this mechanism's "in" variable is linked to 
membrane potential at some point in a cell, the default initial
value stored in the buffer (0) is probably not "correct."
This mechanism can be used with models that can be properly 
initialized in the absence of this mechanism by the simple dodge of 
(a) omitting this mechanism and running a simulation (or at 
least a model initialization) to discover the correct initial 
value for the buffer's contents, and then
(b) restoring this mechanism and specifying the desired value 
for its x0 parameter.

3.  Depending on the variables that are coupled by this mechanism,
delay of 0 may cause instability.

N.T. Carnevale 6/30/2010
ENDCOMMENT


NEURON {
  POINT_PROCESS DLine
  RANGE invl, num, del
  POINTER in, out
  RANGE x0 : initial value to be stored in buffer
  RANGE x : array of sampled values
}

DEFINE MAXNUM 50 : max number of samples availabe to the ring buffer
  : max delay is MAXNUM*invl

PARAMETER {
  invl = 1 (ms) : sample interval
  num = 1 (1) : number of samples in the buffer
  x0 = 0 : initial value stored in the buffer
}

ASSIGNED {
  del (ms) : delay = num*invl
  in (1)
  out (1)
  jin (1) : jin and jout are the indices of the values in the x array
          : that hold the most recent input and output values, respectively
  jout (1)
  x[MAXNUM]
}

INITIAL {
  : first some bulletproofing
  if (num<0) {
    num = 0
  }
  if (num>MAXNUM) {
    num = MAXNUM
  }
  if (invl<0) {
    invl = 0
  }
  : calculate the delay
  del = num*invl
  : initialize the STATEs
  FROM i=0 TO MAXNUM-1 {
    x[i]=x0
  }
  : and the indices
  jout = 0
  jin = jout+num
  : and the "input and output values"
  in = x[jin]
  out = x[jout]
  : finally set up the first sampling
  net_send(0, 1)
}

NET_RECEIVE (w) {
  : respond only to self-events with flag==1
  if (flag==1) {
    x[jin] = in
    out = x[jout]

    jin = jin + 1
    if (jin>=MAXNUM) {
      jin = 0
    }
    jout = jout + 1
    if (jout>=MAXNUM) {
      jout = 0
    }
    net_send(invl, 1)
  }
}
